import { Injectable } from '@angular/core';
import { HttpClient } from '@angular/common/http';
import { BehaviorSubject, Observable, tap } from 'rxjs';
import { ApplicationConfigService } from 'app/core/config/application-config.service';
import { ReviewRequest } from './reviewRequest.model';
import { Review, ReviewStatisticsDTO } from './review.model';
import { ReviewRating } from './reviewRating.model';

@Injectable({ providedIn: 'root' })
export class ReviewManagementService {
  resourceUrl = this.applicationConfigService.getEndpointFor('api/review');

  constructor(private http: HttpClient, private applicationConfigService: ApplicationConfigService) {}

  create(reviewRequest: ReviewRequest): Observable<ReviewRequest> {
    return this.http.post<ReviewRequest>(this.resourceUrl + '/create', reviewRequest).pipe(tap(() => this.resetStatistics()));
  }

  update(review: Review): Observable<Review> {
    return this.http.put<Review>(this.resourceUrl, review).pipe(tap(() => this.resetStatistics()));
  }

  delete(id: number): Observable<{}> {
    return this.http.delete(`${this.resourceUrl}/${id}`).pipe(tap(() => this.resetStatistics()));
  }

  find(id: number): Observable<Review> {
    return this.http.get<Review>(`${this.resourceUrl}/${id}`);
  }

  getAll(): Observable<Review[]> {
    return this.http.get<Review[]>(this.resourceUrl + '/all');
  }

  getAllWithStatistics(): Observable<Review[]> {
    return this.http.get<Review[]>(this.resourceUrl + '/allWithStatistics');
  }

  getAllByUser(): Observable<Review[]> {
    return this.http.get<Review[]>(this.resourceUrl + '/allByUser');
  }

  updateByUser(review: Review): Observable<Review> {
    return this.http.put<Review>(this.resourceUrl + '/updateByUser', review).pipe(tap(() => this.resetStatistics()));
  }

  notifyReviewer(review: Review): Observable<Review> {
    return this.http.post<Review>(this.resourceUrl + '/notifyReviewers', review).pipe(tap(() => this.resetStatistics()));
  }

  rewardBadge(review: Review): Observable<Review> {
    return this.http.post<Review>(this.resourceUrl + '/rewardBadge', review).pipe(tap(() => this.resetStatistics()));
  }

  private reviewStatisticsCache: BehaviorSubject<ReviewStatisticsDTO> | undefined;

  public getReviewStatistics(): Observable<ReviewStatisticsDTO> {
    if (this.reviewStatisticsCache) {
      return this.reviewStatisticsCache;
    } else {
      // no cached value  ->  fetch from server and cache it  ->  return it to subscriber (component) via observable  ->  component updates its view accordingly
      this.reviewStatisticsCache = new BehaviorSubject<ReviewStatisticsDTO>(undefined!);
      this.countAllByUser().subscribe(statistics => {
        if (this.reviewStatisticsCache) {
          this.reviewStatisticsCache.next(statistics); // update cached value
        } else {
          // this should never happen, but just in case
          // we loose all previous listeners to the observable
          this.reviewStatisticsCache = new BehaviorSubject<ReviewStatisticsDTO>(undefined!);
          this.reviewStatisticsCache.next(statistics);
        }
      });
      return this.reviewStatisticsCache;
    }
  }

  /**
   * resets and reloads the review statistics cache
   */
  resetStatistics() {
    this.countAllByUser().subscribe(statistics => {
      if (this.reviewStatisticsCache) {
        this.reviewStatisticsCache.next(statistics); // update cached value
      } else {
        this.reviewStatisticsCache = new BehaviorSubject<ReviewStatisticsDTO>(undefined!);
        this.reviewStatisticsCache.next(statistics);
      }
    });
  }

  private countAllByUser(): Observable<ReviewStatisticsDTO> {
    return this.http.post<ReviewStatisticsDTO>(this.resourceUrl + '/countAllByUser', null);
  }

  getAllReviewRatings(): Observable<ReviewRating[]> {
    return this.http.get<ReviewRating[]>(this.resourceUrl + '/reviewRatings');
  }
}
