import { Component, OnInit } from '@angular/core';
import { HttpResponse } from '@angular/common/http';
import { FormBuilder, Validators } from '@angular/forms';
import { ActivatedRoute } from '@angular/router';
import { Observable } from 'rxjs';
import { finalize, map } from 'rxjs/operators';

import { IWatchListEntry, WatchListEntry } from '../watch-list-entry.model';
import { WatchListEntryService } from '../service/watch-list-entry.service';
import { IUserWatchList } from 'app/entities/user-watch-list/user-watch-list.model';
import { UserWatchListService } from 'app/entities/user-watch-list/service/user-watch-list.service';

@Component({
  selector: 'jhi-watch-list-entry-update',
  templateUrl: './watch-list-entry-update.component.html',
})
export class WatchListEntryUpdateComponent implements OnInit {
  isSaving = false;

  userWatchListsSharedCollection: IUserWatchList[] = [];

  editForm = this.fb.group({
    id: [],
    exerciseId: [null, [Validators.required, Validators.minLength(1), Validators.maxLength(255)]],
    exerciseName: [null, [Validators.maxLength(255)]],
    watchlist: [null, Validators.required],
  });

  constructor(
    protected watchListEntryService: WatchListEntryService,
    protected userWatchListService: UserWatchListService,
    protected activatedRoute: ActivatedRoute,
    protected fb: FormBuilder
  ) {}

  ngOnInit(): void {
    this.activatedRoute.data.subscribe(({ watchListEntry }) => {
      this.updateForm(watchListEntry);

      this.loadRelationshipsOptions();
    });
  }

  previousState(): void {
    window.history.back();
  }

  save(): void {
    this.isSaving = true;
    const watchListEntry = this.createFromForm();
    if (watchListEntry.id !== undefined) {
      this.subscribeToSaveResponse(this.watchListEntryService.update(watchListEntry));
    } else {
      this.subscribeToSaveResponse(this.watchListEntryService.create(watchListEntry));
    }
  }

  trackUserWatchListById(index: number, item: IUserWatchList): number {
    return item.id!;
  }

  protected subscribeToSaveResponse(result: Observable<HttpResponse<IWatchListEntry>>): void {
    result.pipe(finalize(() => this.onSaveFinalize())).subscribe({
      next: () => this.onSaveSuccess(),
      error: () => this.onSaveError(),
    });
  }

  protected onSaveSuccess(): void {
    this.previousState();
  }

  protected onSaveError(): void {
    // Api for inheritance.
  }

  protected onSaveFinalize(): void {
    this.isSaving = false;
  }

  protected updateForm(watchListEntry: IWatchListEntry): void {
    this.editForm.patchValue({
      id: watchListEntry.id,
      exerciseId: watchListEntry.exerciseId,
      exerciseName: watchListEntry.exerciseName,
      watchlist: watchListEntry.watchlist,
    });

    this.userWatchListsSharedCollection = this.userWatchListService.addUserWatchListToCollectionIfMissing(
      this.userWatchListsSharedCollection,
      watchListEntry.watchlist
    );
  }

  protected loadRelationshipsOptions(): void {
    this.userWatchListService
      .query()
      .pipe(map((res: HttpResponse<IUserWatchList[]>) => res.body ?? []))
      .pipe(
        map((userWatchLists: IUserWatchList[]) =>
          this.userWatchListService.addUserWatchListToCollectionIfMissing(userWatchLists, this.editForm.get('watchlist')!.value)
        )
      )
      .subscribe((userWatchLists: IUserWatchList[]) => (this.userWatchListsSharedCollection = userWatchLists));
  }

  protected createFromForm(): IWatchListEntry {
    return {
      ...new WatchListEntry(),
      id: this.editForm.get(['id'])!.value,
      exerciseId: this.editForm.get(['exerciseId'])!.value,
      exerciseName: this.editForm.get(['exerciseName'])!.value,
      watchlist: this.editForm.get(['watchlist'])!.value,
    };
  }
}
